import sux


def test_urlparse():
    up = sux.to_use('urlparse')
    assert "urlparse" == up.__name__
    assert {} == up.parse_qs("")
    assert 'http://www.cwi.nl/%7Eguido/FAQ.html' == up.urljoin('http://www.cwi.nl/%7Eguido/Python.html', 'FAQ.html')



from pytest import raises
import sux


connection = sux.to_use('boto.s3.connection')


def test_connection():
    conn = connection.S3Connection('abc', '123')
    exception_info = raises(Exception, conn.create_bucket, "foo")
    assert "403 Forbidden" in str(exception_info)




"""
Remote execution server, running under python2
"""
from sys import stdin, stdout, version_info
from pickle import loads, dumps, PicklingError

assert version_info.major == 2


counter = iter(xrange(9999999))
_reference_mapping = {}


def process_message(message):
    command = message.pop("command")
    if command == u"import":
        module_name = message.get("module_name")
        result = globals()[module_name] = __import__(
                module_name, fromlist=["foo"])
        return result
    elif command == u"call":
        result = _reference_mapping[message["reference"]](
                *message["args"], **message["kwargs"])
        return result

    elif command == u"getattr":
        if message.get("parent", None) is not None:
            parent = _reference_mapping[message["parent"]]
            return getattr(parent, message["attr"])
        else:
            return globals()[message["attr"]]
    else:
        # unhandled command
        assert False, message


def main():
    finished = False
    while not finished:
        message_length = int(stdin.readline())
        message = stdin.read(message_length)
        try:
            unpickled = loads(message)
            response = process_message(unpickled)
        except Exception as ex:
            # ensure the exception is unpicklable at the other end
            response = Exception(str(ex))
        reference = str(next(counter))
        _reference_mapping[reference] = response
        length = 0
        try:
            payload = dumps(response, protocol=2)
            length = len(payload)
        except PicklingError:
            payload = "*+*+*+"  # invalid pickle format
        stdout.write("{0} {1}\n".format(length, reference))
        if length > 0:
            stdout.write(payload)
        stdout.flush()


if __name__ == '__main__':
    main()



from subprocess import Popen, PIPE
from os import environ
from os.path import join, exists, dirname
from functools import partial
import pickle
from pickle import UnpicklingError
from sys import version_info

assert version_info.major == 3
PYTHON2_VENV = environ["PY2_VIRTUAL_ENV"]
dumps = partial(pickle.dumps, protocol=2, fix_imports=True)
loads = pickle.loads
NoneType = type(None)
_PYTHON2_HELPER_SCRIPT = join(dirname(__file__), "py2helper.py")


class NotAPickle(Exception):
    pass


class Python2Engine:
    def __init__(self, venv):
        python2_exe = join(PYTHON2_VENV, "bin/python")
        assert exists(python2_exe)
        self.process = Popen([python2_exe, _PYTHON2_HELPER_SCRIPT],
                stdin=PIPE, stdout=PIPE)

    def send(self, **kwargs):
        payload = dumps(kwargs)
        self.process.stdin.write("{0}\n".format(len(payload)).encode('utf-8'))
        self.process.stdin.write(payload)
        self.process.stdin.flush()
        line = self.process.stdout.readline()
        message_length, reference = line.decode('utf-8').strip().split(" ")
        message_length = int(message_length)
        if message_length != 0:
            message = self.process.stdout.read(message_length)
        else:
            # only got a reference back; was not picklable at the py2 end
            message = None
        return reference, message


_engine = None


def value_or_reference(command, parent=None, **kwargs):

    reference = None
    pickled = None
    try:
        reference, pickled = _engine.send(command=command,
                                          parent=parent,
                                          **kwargs)
        if pickled is not None:
            unpickled = loads(pickled)
            if isinstance(unpickled, Exception):
                raise unpickled
            else:
                return unpickled
    except (ImportError, UnpicklingError):
        # A valid pickle, but couldn't unpickle
        pass
    assert reference != -1
    return Mock(remote_reference=reference, parent=parent)


class Mock:
    _remote_reference = None

    def __init__(self, remote_reference, parent=None):
        self._parent = parent
        self._remote_reference = remote_reference

    def __call__(self, *args, **kwargs):
        return value_or_reference(
                    command="call",
                    reference=self._remote_reference,
                    parent=self._parent,
                    args=args,
                    kwargs=kwargs)

    def __getattr__(self, attribute_name):
        result = value_or_reference(command="getattr",
                                    parent=self._remote_reference,
                                    attr=attribute_name)
        #setattr(self, attribute_name, result)
        return result


def to_use(module_name, winge=False):
    global _engine
    if _engine is None:
        _engine = Python2Engine(PYTHON2_VENV)
    return value_or_reference(command="import",
                              module_name=module_name)



import os
from setuptools import setup

# Utility function to read the README file.
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "sux",
    version = "0.0.1",
    author = "Nicholas Farrell",
    author_email = "nicholas.farrell@gmail.com",
    description = ("Use python2 packages from python3, if you really have to"),
    license = "BSD",
    keywords = "python2",
    url = "http://packages.python.org/sux",
    packages=['sux', 'tests'],
    install_requires=[
        'setuptools',
    ],
    #long_description=read('README.md'),
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
)
